@@ -1,25 +1,25 @@
 import torch
 
+from torch import Tensor
+from typing import Dict
 from typing import List
 from typing import Optional
 from cftool.types import tensor_dict_type
 
+from .base import MLModel
 from .fcnn import FCNN
 from .linear import Linear
-from ..register import register_ml_module
-from ..schemas.ml import ONE_HOT_KEY
-from ..schemas.ml import EMBEDDING_KEY
-from ..schemas.ml import NUMERICAL_KEY
-from ..schemas.ml import Dimensions
+from ...schema import MLEncoderSettings
+from ...schema import MLGlobalEncoderSettings
 
 
-@register_ml_module("wnd")
-class WideAndDeep(torch.nn.Module):
+@MLModel.register("wnd")
+class WideAndDeep(MLModel):
     def __init__(
         self,
+        input_dim: int,
         output_dim: int,
-        num_history: int,
-        dimensions: Dimensions,
+        num_history: int = 1,
         hidden_units: Optional[List[int]] = None,
         *,
         mapping_type: str = "basic",
@@ -27,10 +27,20 @@ class WideAndDeep(torch.nn.Module):
         activation: str = "ReLU",
         batch_norm: bool = False,
         dropout: float = 0.0,
+        encoder_settings: Optional[Dict[str, MLEncoderSettings]] = None,
+        global_encoder_settings: Optional[MLGlobalEncoderSettings] = None,
     ):
-        super().__init__()
-        wide_dim = dimensions.categorical_dim or dimensions.numerical_dim
-        deep_dim = dimensions.numerical_dim + dimensions.embedding_dim
+        super().__init__(
+            encoder_settings=encoder_settings,
+            global_encoder_settings=global_encoder_settings,
+        )
+        encoder = self.encoder
+        if encoder is None or encoder.is_empty:
+            wide_dim = deep_dim = input_dim
+        else:
+            wide_dim = encoder.categorical_dim
+            numerical_dim = input_dim - encoder.num_one_hot - encoder.num_embedding
+            deep_dim = numerical_dim + encoder.embedding_dim
         self.fcnn = FCNN(
             deep_dim,
             output_dim,
@@ -44,10 +54,13 @@ class WideAndDeep(torch.nn.Module):
         )
         self.linear = Linear(wide_dim, output_dim, num_history, bias=bias)
 
-    def forward(self, batch: tensor_dict_type) -> tensor_dict_type:
-        one_hot = batch[ONE_HOT_KEY]
-        embedding = batch[EMBEDDING_KEY]
-        numerical = batch[NUMERICAL_KEY]
+    def forward(self, net: Tensor) -> tensor_dict_type:
+        if len(net.shape) > 2:
+            net = net.contiguous().view(len(net), -1)
+        encoded = self.encode(net)
+        one_hot = encoded.one_hot
+        embedding = encoded.embedding
+        numerical = encoded.numerical
         # wide
         if one_hot is None and embedding is None:
             wide_inp = numerical
